#  Copyright (c) 2022 Sine Nomine Associates
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to
#  deal in the Software without restriction, including without limitation the
#  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
#  sell copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in
#  all copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
#  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
#  DEALINGS IN THE SOFTWARE.

import os

from molecule import logger
from molecule import util
from molecule.api import Driver


LOG = logger.get_logger(__name__)


class Proxmox(Driver):
    """
    The class responsible for managing instances with Proxmox.

    .. code-block:: yaml

        driver:
          name: proxmox
        platforms:
          - name: instance
            template: generic-centos-8
            memory: 1024
            cpus: 1

    .. code-block:: bash

        $ pip install molecule-proxmox

    """  # noqa

    def __init__(self, config=None):
        super(Proxmox, self).__init__(config)
        self._name = "proxmox"

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, value):
        self._name = value

    @property
    def login_cmd_template(self):
        connection_options = " ".join(self.ssh_connection_options)
        return (
            "ssh {{address}} "
            "-l {{user}} "
            "-p {{port}} "
            "-i {{identity_file}} "
            "{}"
        ).format(connection_options)

    @property
    def default_safe_files(self):
        return []

    @property
    def default_ssh_connection_options(self):
        return self._get_ssh_connection_options()

    def login_options(self, instance_name):
        d = {"instance": instance_name}
        return util.merge_dicts(d, self._get_instance_config(instance_name))

    def ansible_connection_options(self, instance_name):
        try:
            d = self._get_instance_config(instance_name)
            return {
                "ansible_user": d["user"],
                "ansible_host": d["address"],
                "ansible_port": d["port"],
                "ansible_private_key_file": d["identity_file"],
                "connection": "ssh",
                "ansible_ssh_common_args": " ".join(self.ssh_connection_options),
            }
        except StopIteration:
            return {}
        except IOError:
            # Instance has yet to be provisioned, therefore the
            # instance_config is not on disk.
            return {}

    def _get_instance_config(self, instance_name):
        instance_config_dict = util.safe_load_file(self._config.driver.instance_config)
        return next(
            item for item in instance_config_dict if item["instance"] == instance_name
        )

    def sanity_checks(self):
        pass

    def template_dir(self):
        """Return path to its own cookiecutterm templates. It is used by init
        command in order to figure out where to load the templates from.
        """
        return os.path.join(os.path.dirname(__file__), "cookiecutter")

    def modules_dir(self):
        return os.path.join(os.path.dirname(__file__), "modules")



#!/usr/bin/python

# Copyright (c) 2022, Sine Nomine Associates
# BSD 2-Clause License

ANSIBLE_METADATA = {
    'metadata_version': '1.1.',
    'status': ['preview'],
    'supported_by': 'community',
}

DOCUMENTATION = r"""
---
module: proxmox_qemu_agent

short_description: todo

description:
  - todo
requirements:

options:

author:
  - Michael Meffie (@meffie)
"""

EXAMPLES = r"""
"""

RETURN = r"""
"""

import time
import syslog
import pprint

from proxmoxer import ProxmoxAPI
from ansible.module_utils.basic import AnsibleModule


def get_vm(module, proxmox, vmid):
    vm_list = [vm for vm in proxmox.cluster.resources.get(type='vm') if vm['vmid'] == int(vmid)]
    if len(vm_list) == 0:
        module.fail_json(vmid=vmid, msg='VM with vmid = %s not found' % vmid)
    if len(vm_list) > 1:
        module.fail_json(vmid=vmid, msg='Multiple VMs with vmid = %s found' % vmid)
    return vm_list[0]


def get_addresses(interfaces):
    """
    Extract the non-loopback IPv4 addresses from network-get-interfaces results.

    Example:
        interfaces = [
            {'hardware-address': '6e:25:bb:c7:4b:76',
             'name': 'ens18',
             'ip-addresses': [
                {'ip-address': '192.168.136.176', 'ip-address-type': 'ipv4', 'prefix': 24},
                {'ip-address': 'fe80::6c25:bbff:fec7:4b76', 'ip-address-type': 'ipv6', 'prefix': 64},
             ]
             ...
            },
            {'hardware-address': '00:00:00:00:00:00',
             'name': 'lo',
            ...
        ]}]

        get_addresses(interface)
        ['192.168.136.85']

    """
    addrs = []
    for interface in interfaces:
        if 'ip-addresses' in interface:
            for ip_address in interface['ip-addresses']:
                atype = ip_address.get('ip-address-type', '')
                aip = ip_address.get('ip-address', '')
                if aip and atype == 'ipv4' and not aip.startswith('127.'):
                    addrs.append(aip)
    return addrs


def run_module():
    """
    todo
    """
    result = dict(
        changed=False,
    )
    module = AnsibleModule(
        argument_spec=dict(
            api_host=dict(type='str', required=True),
            api_user=dict(type='str', required=True),
            api_password=dict(type='str', no_log=True),
            api_token_id=dict(type='str', no_log=True),
            api_token_secret=dict(type='str', no_log=True),
            validate_certs=dict(type='bool', default=False),
            vmid=dict(type='int', required=True),
        ),
        required_together=[('api_token_id', 'api_token_secret')],
        required_one_of=[('api_password', 'api_token_id')],
    )
    api_host = module.params['api_host']
    validate_certs = module.params['validate_certs']
    api_user = module.params['api_user']
    api_password = module.params['api_password']
    api_token_id = module.params['api_token_id']
    api_token_secret = module.params['api_token_secret']
    vmid = module.params['vmid']

    auth_args = {'user': api_user}
    if not (api_token_id and api_token_secret):
        auth_args['password'] = api_password
    else:
        auth_args['token_name'] = api_token_id
        auth_args['token_value'] = api_token_secret

    proxmox = ProxmoxAPI(api_host, verify_ssl=validate_certs, **auth_args)

    retries = 20
    while retries > 0:
        try:
            vm = get_vm(module, proxmox, vmid)
            result['vm'] = vm
            reply = proxmox.nodes(vm['node']).qemu(vmid).agent.get('network-get-interfaces')
            syslog.syslog("reply: %s" % pprint.pformat(reply, compact=True))
            if 'result' in reply:
                interfaces = reply['result']
                addresses = get_addresses(interfaces)
                if addresses:
                    result['vm']['interfaces'] = interfaces
                    result['vm']['addresses'] = addresses
                    break
        except Exception as e:
            syslog.syslog("Exception type %s" % type(e))
            syslog.syslog(str(e))
            pass
        retries -= 1
        time.sleep(5)

    module.exit_json(**result)


def main():
    run_module()


if __name__ == '__main__':
    main()



import setuptools
import re

name = 'molecule-proxmox'
description='Proxmox Molecule Plugin :: run molecule tests using proxmox'

def find_version():
    text = open('src/%s/__init__.py' % name.replace('-', '_')).read()
    return re.search(r"__version__\s*=\s*'(.*)'", text).group(1)

setuptools.setup(
    name=name,
    version=find_version(),
    author='Michael Meffie',
    author_email='mmeffie@sinenomine.net',
    description=description,
    long_description=open('README.rst').read(),
    long_description_content_type='text/x-rst',
    url='https://github.com/meffie/molecule-proxmox',
    packages=setuptools.find_packages(where='src'),
    package_dir={'': 'src'},
    include_package_data=True,
    entry_points={
        'molecule.driver': [
            'proxmox = molecule_proxmox.driver:Proxmox',
        ],
    },
    install_requires=[
        # molecule plugins are not allowed to mention Ansible as a direct dependency
        'molecule>=3.2.0',
        'pyyaml>=5.1,<6',
        'proxmoxer',
    ],
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.6',
)
